#/bin/bash
# TanaT le 29 juin 2009

# ---------------------------------------------------------------
# Settings
# ---------------------------------------------------------------

MAXSIZE=+90k            # See -size argument in "find" man
                        # Example MAXSIZE=+90k (file size > 90ko)

SQLFILE=update_commands.sql

DISTFOLD=IMG            # Root folder of your image folders
                        # SPIP website, set DISTFOLD=IMG

                                # Database settings
YOURDB=le_pave_dans_la_mare     # DB name
TABLENAME=pub_documents         # SPIP ex : pub_documents
SIZEATTR=taille                 # SPIP ex : taille   
WIDTHATTR=largeur               # SPIP ex : largeur 
HEIGHTATTR=hauteur              # SPIP ex : hauteur
FILEATTR=fichier                # SPIP ex : fichier  

MAXRESW=1024
MAXRESH=768
MAXQUAL=80    # To do...

# ---------------------------------------------------------------
# Functions
# ---------------------------------------------------------------

function usage()
{
    cat << FIN_MAN
Usage :

$(basename $0) [ -h ] [ -s sql_file ] [ -W width ] [ -H height ] go files_folder
$(basename $0) reduces all images.jpg found in files_folder if their \
size exceed $MAXSIZE and their resolution doesn't fit in $SETRES.
$(basename $0) also creates a sql commands file to update your SPIP \
database with new size, width and height of images reduced.

Options :

    -h           print this manual page.
    -s sql_file  specify a filename into write sql commands. Default one \
created is $SQLFILE.
    -W width     size (px) max. If image bigger then it's reduced. Default is $MAXRESW.
    -H height    size (px) max. If image bigger then it's reduced. Default is $MAXRESH.

Caution :

Be aware that original images will be resized permanently. Backup files_folder first \
if you want to.

Notes :

1. SPIP website : files_folder should be "jpg" for instance : Launch this script \
in a folder containing files_folder.
2. "go" argument prevent to launch this script carelessly !
FIN_MAN
    exit 0
}

function die()
{
    echo $1
    exit 1
}

# ---------------------------------------------------------------
# Options and args
# ---------------------------------------------------------------

while getopts hs:W:H: option
do
    case $option in
        h) usage ;;
        s) SQLFILE=$(echo "$OPTARG" | tr " " "_") ;;
        W) MAXRESW=$OPTARG ;;
        H) MAXRESH=$OPTARG ;;
        ?) usage ;;
    esac
done

shift $(($OPTIND - 1))  # Args index shifting

# If not exactly 2 arg, echo usage
[ "$#" -ne "2" ] && usage || :

# If 1st arg not "go", echo usage
[ "$1" != "go" ] && usage || :

# Else set IMGFOLD as second arg
IMGFOLD=$2

# Resolution pattern
SETRES=${MAXRESW}x${MAXRESH}
# Quality required
SETQUAL=$MAXQUAL

# ---------------------------------------------------------------
# Fallback
# ---------------------------------------------------------------

IDENTIFY=$(which identify)
[ $? -eq 1 ] && die "Command identify not found. Please install ImageMagick package." || :
MOGRIFY=$(which mogrify)
[ $? -eq 1 ] && die "Command mogrify not found. Please install ImageMagick package." || :

# ---------------------------------------------------------------
# Main
# ---------------------------------------------------------------

for file in $(find $IMGFOLD -iname '*.jpg' -size $MAXSIZE)
do
    fileresol=$($IDENTIFY $file | grep -Eo "[[:digit:]]*x[[:digit:]]*" | tail -1)
    filew=${fileresol%%x*}
    fileh=${fileresol##*x}
    
    if [ "$fileh" -gt "$filew" ] ; then
        # On inverse h et w pour que le test suivant soit ok (il faut w > h)
        tmpsize=$fileh
        fileh=$filew
        filew=$tmpsize
    fi

        echo treating $file...
    if [ "$filew" -gt "$MAXRESW" ] || [ "$fileh" -gt "$MAXRESH" ] ; then
        echo "     -- w:$filew h:$fileh -> Decreasing resolution..."
        $MOGRIFY -resize $SETRES -quality $SETQUAL $file

        # on met a jour les nouvelles $SIZEATTR et resolution
        fileresol=$($IDENTIFY $file | grep -Eo "[[:digit:]]*x[[:digit:]]*" | tail -1)
        filew=${fileresol%%x*}
        fileh=${fileresol##*x}
        fileoctets=$(stat -c %s $file)

        echo "     -- Updating sql file..."
        cat >> $SQLFILE <<_EOF_
UPDATE \`$YOURDB\`.\`$TABLENAME\` SET \`$SIZEATTR\` = '$fileoctets',
\`$WIDTHATTR\` = '$filew',
\`$HEIGHTATTR\` = '$fileh' WHERE \`$TABLENAME\`.\`$FILEATTR\` = '$DISTFOLD/$file' LIMIT 1 ;
_EOF_
#    elif [] ; then
#       echo truc
    fi 
done

exit 0
